/*
 * JBoss, the OpenSource J2EE webOS
 *
 * Distributable under LGPL license.
 * See terms of license at gnu.org.
 */

// $Id: MBeanAttributeInfoSer.java,v 1.4.6.1 2004/08/08 21:28:04 tdiesler Exp $

package org.jboss.net.jmx.adaptor;

import org.apache.axis.Constants;
import org.apache.axis.encoding.SerializationContext;
import org.apache.axis.encoding.Serializer;
import org.apache.axis.wsdl.fromJava.Types;
import org.w3c.dom.Element;
import org.xml.sax.Attributes;

import javax.management.MBeanAttributeInfo;
import javax.xml.namespace.QName;
import java.io.IOException;

/**
 * An <code>MBeanAttributeInfoSer</code> is be used to serialize
 * MBeanAttributeInfo using the <code>SOAP-ENC</code>
 * encoding style.<p>
 *
 * @author <a href="mailto:a_taherkordi@users.sourceforge.net">Alireza Taherkordi</a>
 * @version $Revision: 1.4.6.1 $
 */
public class MBeanAttributeInfoSer implements Serializer
{

   public void serialize(
      QName name,
      Attributes attributes,
      Object value,
      SerializationContext context)
      throws IOException
   {
      if (!(value instanceof MBeanAttributeInfo))
         throw new IOException(
            "Can't serialize a "
            + value.getClass().getName()
            + " instance with a MBeanAttributeInfo Serializer.");
      context.startElement(name, attributes);

      MBeanAttributeInfo info = (MBeanAttributeInfo) value;
      context.serialize(new QName("", "name"), null, info.getName());
      context.serialize(new QName("", "type"), null, info.getType());
      context.serialize(
         new QName("", "description"),
         null,
         info.getDescription());
      context.serialize(
         new QName("", "isReadable"),
         null,
         new Boolean(info.isReadable()));
      context.serialize(
         new QName("", "isWritable"),
         null,
         new Boolean(info.isWritable()));
      context.serialize(new QName("", "isIs"), null, new Boolean(info.isIs()));

      context.endElement();
      return;

   }

   public Element writeSchema(Class aClass, Types types) throws Exception
   {
      return null;
   }

   public String getMechanismType()
   {
      return Constants.AXIS_SAX;
   }

}